<?php
class BarChart extends Chart {
  
  var $_font_position;
  
  function BarChart($width, $height, $sections=Array()) {
    $this->_width = (int)$width;
    $this->_height = (int)$height;
    $this->_sections = (array)$sections;
    
    $this->setBackgroundColor('transparent');
    $this->setBorderColor('FFFFFF');
    $this->setBorderWidth(1);
    $this->setFontSize(3);
	
    $this->setFontPosition('inside');
  }
  
  function setFontPosition($font_position) { // top or inside
    $this->_font_position = strtolower($font_position);
    
    return true;
  }
  
  function generate() {
    // initialize some vars
    $image_center_x = $this->_width / 2;
    $image_center_y = $this->_height / 2;
    $uses_alpha = false;
    if (empty($this->_background_color) || strtolower($this->_background_color) == 'transparent') {
      $uses_alpha = true;
    }
    
    // initialize image
    $barchart_image = imagecreatetruecolor($this->_width, $this->_height);
    
    // draw background
    if ($uses_alpha) {
      imagealphablending($barchart_image, false);
      $background_color = imagecolorallocatealpha($barchart_image, 0, 0, 0, 127);
      imagefill($barchart_image, 0, 0, $background_color);
      imagealphablending($barchart_image, true);
    } else {
      $background_color_array = $this->convertRGB($this->_background_color);
      $background_color = imagecolorallocate($barchart_image, $background_color_array['red'], $background_color_array['green'], $background_color_array['blue']);
      imagefill($barchart_image, 0, 0, $background_color);
    }
    
    // organize bars in descending order
    $sort_key = 0;
    $sections_sorted = $this->multiSortOnValue($this->_sections, $sort_key);
    
    // set border color
    $border_color_array = $this->convertRGB($this->_border_color);
    $border_color = imagecolorallocate($barchart_image, $border_color_array['red'], $border_color_array['green'], $border_color_array['blue']);
    
    // draw bars
    $font_height = imagefontheight($this->_font_size);
    $total_spacing = $this->_border_width * (count($this->_sections));
    $bar_max_width = $this->_width - $this->_border_width;
    $bar_max_size = $sections_sorted[0][0];
    $bar_x = 0;
    $bar_y = 0;
    $color_index = 0;
    $bar_colors = Array();
    
    switch ($this->_font_position) {
      case 'top':
        $bar_height = (($this->_height - $total_spacing) / count($this->_sections));
        $bar_height -= (count($this->_sections) * $font_height) / count($this->_sections);
        $bar_y = $font_height;
        break;
      case 'inside':
      default:
        $bar_height = (($this->_height - $total_spacing) / count($this->_sections));
        break;
    }
    
    foreach ($this->_sections as $bar) {
      $bar_size = $bar[0];
      $bar_color = $bar[1];
      
      $bar_width = ($bar_max_width / $bar_max_size) * $bar_size;
      
      if ($this->_border_width > 0) {
        imagefilledrectangle($barchart_image, $bar_x, $bar_y, $bar_width + $bar_x, $bar_height + $bar_y, $border_color);
      }
      
      $bar_color_array = $this->convertRGB($bar_color);
      $bar_colors[$color_index] = imagecolorallocate($barchart_image, $bar_color_array['red'], $bar_color_array['green'], $bar_color_array['blue']);
      imagefilledrectangle($barchart_image, $bar_x + $this->_border_width, $bar_y + $this->_border_width, $bar_width + $bar_x - $this->_border_width, $bar_height + $bar_y - $this->_border_width, $bar_colors[$color_index]);
      
      switch ($this->_font_position) {
        case 'top':
          $bar_y += $bar_height + $font_height + ($this->_border_width);
          break;
        case 'inside':
        default:
          $bar_y += $bar_height + $this->_border_width;
          break;
      }
      
      $color_index++;
    }
    
    if ($uses_alpha) {
      imagealphablending($barchart_image, false);
      imagesavealpha($barchart_image, true);
    }
    
    // draw text
    $color_index = 0;
    $text_colors = Array();
    $bar_y = 0;
    foreach ($this->_sections as $bar) {
      $text = $bar[2];
      $text_color = $bar[3];
      
      $text_color_array = $this->convertRGB($text_color);
      $text_colors[$color_index] = imagecolorallocate($barchart_image, $text_color_array['red'], $text_color_array['green'], $text_color_array['blue']);
      
      $text_x = max(2, $this->_border_width * 2);
      
      switch ($this->_font_position) {
        case 'top':
          $text_y = $bar_y;
          break;
        case 'inside':
        default:
          $text_y = $bar_y + ($bar_height / 2) - ($font_height / 2);
          break;
      }
      
      imagestring($barchart_image, $this->_font_size, $text_x, $text_y, $text, $text_colors[$color_index]);
      
      $color_index++;
      
      switch ($this->_font_position) {
        case 'top':
          $bar_y += $bar_height + $font_height + $this->_border_width;
          break;
        case 'inside':
        default:
          $bar_y += $bar_height + $this->_border_width;
          break;
      }
    }
    
    if ($uses_alpha) {
      imagealphablending($barchart_image, false);
      imagesavealpha($barchart_image, true);
    }
    
    header('Content-type: image/png');
    imagepng($barchart_image);
    imagedestroy($barchart_image);
  }
  
}
?>
